# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class BcsXpSegmentH(HeaderSegment):

    sender = HprimMultiple(
        5,
        name=HprimString(1, default="MCA"),
        software_version=HprimString(2),
        serial_number=HprimString(4),
        interface_version=HprimString(5),
    )

    # P: Production
    # Q: QC
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class BcsXpSegmentP(Segment):

    patient_id = HprimString(3)
    laboratory_patient_id = HprimString(4)
    instrument_patient_id = HprimString(5)
    last_name = HprimString((6, 1), max_lenght=20, trunc=True)
    first_name = HprimString((6, 2), max_lenght=20, trunc=True)
    middle_name = HprimString((6, 3))
    birthday = HprimDate(8, fmt_date="%Y%m%d")

    # M/F/U or blank
    sex = HprimString(9)
    orders = HprimForeign("O", "patient")


class BcsXpSegmentQ(Segment):

    sample_id = HprimString((3, 2))
    tests = HprimString((5, 4), default='ALL')
    # O ou X pour order vide
    status_code = HprimString(13)


class BcsXpSegmentO(Segment):

    sample_id = HprimString(3)
    sample_id2 = HprimString(4)
    assay = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        name=HprimString(5),
    ))

    # S: Stat
    # R: Routine
    priority = HprimString(6)

    # Q: QC sinon vide
    action_code = HprimString(12)
    physician = HprimString(17)

    # F : resultat final
    # X : errors
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class BcsXpSegmentR(Segment):

    assay = HprimMultiple(
        3,
        code=HprimString(4),
        name=HprimString(5),
        dilution=HprimString(6),
        # Always F
        status=HprimString(7),
    )
    result = HprimString(4)
    unit = HprimString(5)
    ranges = HprimString(6)

    # L: Less than normal range
    # H: Higher than normal range
    # <: below measuring range
    # >: above measuring range
    # W: worse, i.e. no clot
    # A: abnormal, i.e. invalid
    abnormal_flag = HprimString(7)

    # F: Final results
    # R: Repeat
    resu_status = HprimString(9)
    patient_id = HprimString(11)

    # date du resultat
    date = HprimDate(13)


class BcsXpSegmentC(Segment):

    # I: Instrument
    source = HprimString(3)
    text = HprimString(4)

    # G: result flag
    # I: Instrument exception string
    comment_type = HprimString(5)


class BcsXpSegmentL(Segment):
    pass


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bcsxp",
        "label": "BCS XP/Atellica Coag 360 (ASTM)",
    }
    SEGMENT_CLASS_MAP = {
        "H": BcsXpSegmentH,
        "P": BcsXpSegmentP,
        "O": BcsXpSegmentO,
        "Q": BcsXpSegmentQ,
        "R": BcsXpSegmentR,
        "C": BcsXpSegmentC,
        "L": BcsXpSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = BcsXpSegmentH(type_segment="H", **kwargs)
        message.end = BcsXpSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def encode_worklist(self, worklist, context):
        message = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                laboratory_patient_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                middle_name="",
                birthdate=patient.birthdate,
                sex=patient.sex,
            )

            for sample, action, priority in patient.samples:
                physician = [p for p in sample.physicians if p.type in ("prescriber")]
                seg_o = seg_p.orders.add(
                    sample_id2=sample.sample_id,
                    priority=priority,
                    code_action=action,
                    physician=physician[0].code if physician else "",
                    report_type="O",
                )
                for order in sample.orders:
                    t = seg_o.assay.add()
                    t.code = order["code"]

        if message.patients:
            return message
        elif context["query"]:
            for seg_q in context["query"].queries:
                seg_noreq = message.queries.add()
                seg_noreq.sample_id = seg_q.sample_id
                seg_noreq.status_code = 'X'
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.action_code == "Q":
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_id,
                            control_lot=seg_o.sample_id,
                            type="control"
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_id,
                            type="patient"
                        )
                    for seg_r in seg_o.results:
                        alarmes = []
                        result = sample.results.add(
                            code=seg_r.assay.code,
                            value=seg_r.result,
                            date=seg_r.date,
                            dilution=seg_r.assay.dilution,
                        )
                        if seg_r.abnormal_flag:
                            alarmes.append(seg_r.abnormal_flag)
                        commentaires = []
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_type == "I":
                                commentaires.append(seg_c.text)
                            if seg_c.comment_type == "G":
                                alarmes.append(seg_c.text)
                        if alarmes:
                            result.flags = alarmes
                        if commentaires:
                            result.comments = commentaires
        return worklist
